@@ -103,7 +103,7 @@ class Common_services(extensions.ExtensionDescriptor):
 
     @classmethod
     def get_namespace(cls):
-        return 'http://wiki.openstack.org/Tacker'
+        return 'https://wiki.openstack.org/Tacker'
 
     @classmethod
     def get_updated(cls):
