@@ -19,6 +19,7 @@ from oslo_log import log as logging
 from tacker._i18n import _
 from tacker.common.container import kubernetes_utils
 from tacker.common import log
+from tacker import context as t_context
 from tacker.extensions import nfvo
 from tacker.keymgr import API as KEYMGR_API
 from tacker.nfvo.drivers.vim import abstract_vim_driver
@@ -30,7 +31,7 @@ KUBERNETES_OPTS = [
     cfg.BoolOpt('use_barbican', default=True,
                 help=_('Use barbican to encrypt vim password if True'
                        ', save vim credentials in local file system'
-                       'if False'))
+                       ' if False'))
 ]
 cfg.CONF.register_opts(KUBERNETES_OPTS, 'k8s_vim')
 
@@ -74,8 +75,8 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
         return auth_cred, file_descriptor
 
     def _create_ssl_ca_file(self, auth_cred):
-        ca_cert = auth_cred['ssl_ca_cert']
-        if ca_cert is not None:
+        if auth_cred.get('ssl_ca_cert', ''):
+            ca_cert = auth_cred['ssl_ca_cert']
             file_descriptor, file_path = \
                 self.kubernetes.create_ca_cert_tmp_file(ca_cert)
             auth_cred['ca_cert_file'] = file_path
@@ -127,7 +128,7 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
             self.kubernetes.close_tmp_file(file_descriptor, file_path)
 
     @log.log
-    def register_vim(self, context, vim_obj):
+    def register_vim(self, vim_obj):
         """Validate Kubernetes VIM."""
         if 'key_type' in vim_obj['auth_cred']:
             vim_obj['auth_cred'].pop(u'key_type')
@@ -135,21 +136,21 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
             vim_obj['auth_cred'].pop(u'secret_uuid')
         self.authenticate_vim(vim_obj)
         self.discover_placement_attr(vim_obj)
-        self.encode_vim_auth(context, vim_obj['id'],
+        self.encode_vim_auth(vim_obj['id'],
                              vim_obj['auth_cred'])
         LOG.debug('VIM registration completed for %s', vim_obj)
 
     @log.log
-    def deregister_vim(self, context, vim_obj):
+    def deregister_vim(self, vim_obj):
         """Deregister Kubernetes VIM from NFVO
 
         Delete VIM keys from file system
         """
-        self.delete_vim_auth(context, vim_obj['id'],
+        self.delete_vim_auth(vim_obj['id'],
                              vim_obj['auth_cred'])
 
     @log.log
-    def delete_vim_auth(self, context, vim_id, auth):
+    def delete_vim_auth(self, vim_id, auth):
         """Delete kubernetes vim information
 
         Delete vim key stored in file system
@@ -160,10 +161,12 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
                       vim_id)
             if auth.get('key_type') == 'barbican_key':
                 try:
+                    k_context = \
+                        t_context.generate_tacker_service_context()
                     keystone_conf = CONF.keystone_authtoken
                     secret_uuid = auth['secret_uuid']
                     keymgr_api = KEYMGR_API(keystone_conf.auth_url)
-                    keymgr_api.delete(context, secret_uuid)
+                    keymgr_api.delete(k_context, secret_uuid)
                     LOG.debug('VIM key deleted successfully for vim %s',
                               vim_id)
                 except Exception as exception:
@@ -175,7 +178,7 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
                 raise nfvo.VimEncryptKeyError(vim_id=vim_id)
 
     @log.log
-    def encode_vim_auth(self, context, vim_id, auth):
+    def encode_vim_auth(self, vim_id, auth):
         """Encode VIM credentials
 
          Store VIM auth using fernet key encryption
@@ -189,16 +192,17 @@ class Kubernetes_Driver(abstract_vim_driver.VimAbstractDriver):
             encoded_auth = fernet_obj.encrypt(
                 auth['bearer_token'].encode('utf-8'))
             auth['bearer_token'] = encoded_auth
-        if ('ssl_ca_cert' in auth) and (auth['ssl_ca_cert'] is not None):
+        if auth.get('ssl_ca_cert', ''):
             encoded_auth = fernet_obj.encrypt(
                 auth['ssl_ca_cert'].encode('utf-8'))
             auth['ssl_ca_cert'] = encoded_auth
 
         if CONF.k8s_vim.use_barbican:
             try:
+                k_context = t_context.generate_tacker_service_context()
                 keystone_conf = CONF.keystone_authtoken
                 keymgr_api = KEYMGR_API(keystone_conf.auth_url)
-                secret_uuid = keymgr_api.store(context, fernet_key)
+                secret_uuid = keymgr_api.store(k_context, fernet_key)
 
                 auth['key_type'] = 'barbican_key'
                 auth['secret_uuid'] = secret_uuid
