@@ -43,6 +43,7 @@ from tacker.keymgr import API as KEYMGR_API
 from tacker import manager
 from tacker.nfvo.workflows.vim_monitor import vim_monitor_utils
 from tacker.plugins.common import constants
+from tacker.vnfm import keystone
 from tacker.vnfm import vim_client
 
 from tacker.tosca import utils as toscautils
@@ -87,10 +88,22 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
             cfg.CONF.nfvo_vim.vim_drivers)
         self.vim_client = vim_client.VimClient()
 
+    @staticmethod
+    def validate_keystone_auth_url(auth_url, verify):
+        keystone_obj = keystone.Keystone()
+        auth_url = utils.get_auth_url_v3(auth_url)
+        try:
+            return keystone_obj.get_version(auth_url, verify)
+        except Exception as e:
+            LOG.error('Keystone Auth URL invalid')
+            raise nfvo.VimConnectionException(message=str(e))
+
     def get_auth_dict(self, context):
         auth = CONF.keystone_authtoken
+        auth_url = utils.get_auth_url_v3(auth.auth_url)
+        self.validate_keystone_auth_url(auth_url, 'True')
         return {
-            'auth_url': auth.auth_url + '/v3',
+            'auth_url': auth_url,
             'token': context.auth_token,
             'project_domain_name': auth.project_domain_name or context.domain,
             'project_name': context.tenant_name
@@ -104,6 +117,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         LOG.debug('Create vim called with parameters %s',
                   strutils.mask_password(vim))
         vim_obj = vim['vim']
+        vim_obj['auth_url'] = utils.get_auth_url_v3(vim_obj['auth_url'])
         vim_type = vim_obj['type']
         vim_obj['id'] = uuidutils.generate_uuid()
         vim_obj['status'] = 'PENDING'
@@ -332,11 +346,13 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
             name_match_list.append(classifier_dict)
         # grab the first VNF to check it's VIM type
         # we have already checked that all VNFs are in the same VIM
-        vim_obj = self._get_vim_from_vnf(context,
-                                         list(vnffg_dict[
-                                              'vnf_mapping'].values())[0])
+        vim_obj = self._get_vim_from_vnf(
+            context, list(vnffg_dict['vnf_mapping'].values())[0])
         # TODO(trozet): figure out what auth info we actually need to pass
         # to the driver.  Is it a session, or is full vim obj good enough?
+
+        correlation = super(NfvoPlugin, self)._get_correlation_template(
+            context, vnffg_info)
         driver_type = vim_obj['type']
         try:
             fc_ids = []
@@ -353,6 +369,7 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
                                               vnfs=sfc['chain'],
                                               fc_ids=fc_ids,
                                               symmetrical=sfc['symmetrical'],
+                                              correlation=correlation,
                                               auth_attr=vim_obj['auth_cred'])
         except Exception:
             with excutils.save_and_reraise_exception():
@@ -552,17 +569,19 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         f = fernet.Fernet(vim_key)
         if not f:
             LOG.warning('Unable to decode VIM auth')
-            raise nfvo.VimNotFoundException(
-                'Unable to decode VIM auth key')
+            raise nfvo.VimNotFoundException(vim_id=vim_id)
         return f.decrypt(cred)
 
     @staticmethod
     def _find_vim_key(vim_id):
         key_file = os.path.join(CONF.vim_keys.openstack, vim_id)
         LOG.debug('Attempting to open key file for vim id %s', vim_id)
-        with open(key_file, 'r') as f:
-            return f.read()
-        LOG.warning('VIM id invalid or key not found for  %s', vim_id)
+        try:
+            with open(key_file, 'r') as f:
+                    return f.read()
+        except Exception:
+            LOG.warning('VIM id invalid or key not found for  %s', vim_id)
+            raise nfvo.VimKeyNotFoundException(vim_id=vim_id)
 
     def _vim_resource_name_to_id(self, context, resource, name, vnf_id):
         """Converts a VIM resource name to its ID
@@ -608,7 +627,11 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         LOG.debug('nsd_dict: %s', inner_nsd_dict)
 
         vnfm_plugin = manager.TackerManager.get_service_plugins()['VNFM']
-        vnfd_imports = inner_nsd_dict['imports']
+        vnfd_imports = inner_nsd_dict.get('imports')
+        if not vnfd_imports:
+            LOG.error('VNFD import section is missing')
+            raise nfvo.ToscaParserFailed(
+                error_msg_details='VNFD import section is missing')
         inner_nsd_dict['imports'] = []
         new_files = []
         for vnfd_name in vnfd_imports:
@@ -712,8 +735,9 @@ class NfvoPlugin(nfvo_db_plugin.NfvoPluginDb, vnffg_db.VnffgPluginDbMixin,
         nsd_dict = yaml.safe_load(nsd['attributes']['nsd'])
         vnfm_plugin = manager.TackerManager.get_service_plugins()['VNFM']
         onboarded_vnfds = vnfm_plugin.get_vnfds(context, [])
-        region_name = ns.setdefault('placement_attr', {}).get(
-            'region_name', None)
+        region_name = ns_info.get('placement_attr', {}).\
+            get('region_name', None)
+
         vim_res = self.vim_client.get_vim(context, ns['ns']['vim_id'],
                                           region_name)
         driver_type = vim_res['vim_type']
