@@ -15,6 +15,7 @@
 import time
 import yaml
 
+from blazarclient import client as blazar_client
 from glanceclient.v2 import client as glance_client
 from keystoneauth1.identity import v3
 from keystoneauth1 import session
@@ -106,6 +107,21 @@ class BaseTackerTest(base.BaseTestCase):
         data['project_domain_name'] = domain_name
         return clients.OpenstackClients(auth_attr=data).heat
 
+    def blazarclient(cls):
+        data = cls.get_credentials()
+        domain_name = data.pop('domain_name')
+        data['user_domain_name'] = domain_name
+        data['project_domain_name'] = domain_name
+        auth_ses = clients.OpenstackClients(auth_attr=data).keystone_session
+        args = {
+            'session': auth_ses,
+            'service_type': 'reservation',
+            'interface': 'public',
+            'region_name': 'RegionOne',
+        }
+        client = blazar_client.Client(**args)
+        return client
+
     @classmethod
     def glanceclient(cls):
         vim_params = cls.get_credentials()
@@ -119,6 +135,9 @@ class BaseTackerTest(base.BaseTestCase):
         auth_ses = session.Session(auth=auth, verify=verify)
         return glance_client.Client(session=auth_ses)
 
+    def get_vdu_resource(self, stack_id, res_name):
+        return self.h_client.resources.get(stack_id, res_name)
+
     def wait_until_vnf_status(self, vnf_id, target_status, timeout,
                               sleep_interval):
         start_time = int(time.time())
@@ -140,6 +159,17 @@ class BaseTackerTest(base.BaseTestCase):
         self.wait_until_vnf_status(vnf_id, 'ACTIVE', timeout,
                                    sleep_interval)
 
+    def verify_vnf_update(self, vnf_id):
+        self.wait_until_vnf_status(vnf_id, 'ACTIVE',
+                                   constants.VNF_CIRROS_CREATE_TIMEOUT,
+                                   constants.ACTIVE_SLEEP_TIME)
+        self.wait_until_vnf_status(vnf_id, 'PENDING_HEAL',
+                                   constants.VNF_CIRROS_PENDING_HEAL_TIMEOUT,
+                                   constants.PENDING_SLEEP_TIME)
+        self.wait_until_vnf_status(vnf_id, 'ACTIVE',
+                                  constants.VNF_CIRROS_CREATE_TIMEOUT,
+                                  constants.ACTIVE_SLEEP_TIME)
+
     def wait_until_vnf_delete(self, vnf_id, timeout):
         start_time = int(time.time())
         while True:
@@ -171,7 +201,8 @@ class BaseTackerTest(base.BaseTestCase):
             constants.VNF_CIRROS_CREATE_TIMEOUT,
             constants.ACTIVE_SLEEP_TIME)
         self.validate_vnf_instance(vnfd_instance, vnf_instance)
-        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf']['mgmt_url'])
+        self.assertIsNotNone(self.client.show_vnf(vnf_id)['vnf'][
+            'mgmt_ip_address'])
 
         self.wait_until_vnf_dead(
             vnf_id,
@@ -237,3 +268,14 @@ class BaseTackerTest(base.BaseTestCase):
                                               resource_name=resource_name)
         resource_dict = resource_details.to_dict()
         self.assertTrue(resource_dict['attributes']['port_security_enabled'])
+
+    def trigger_vnf(self, vnf, policy_name, policy_action):
+        credential = 'g0jtsxu9'
+        body = {"trigger": {'policy_name': policy_name,
+                            'action_name': policy_action,
+                            'params': {
+                                'data': {'alarm_id': '35a80852-e24f-46ed-bd34-e2f831d00172', 'current': 'alarm'},  # noqa
+                                'credential': credential}
+                            }
+                }
+        self.client.post('/vnfs/%s/triggers' % vnf, body)
