@@ -19,9 +19,12 @@ import yaml
 
 from collections import OrderedDict
 from oslo_log import log as logging
+from oslo_utils import uuidutils
+from tacker.common import exceptions
 from tacker.common import log
 from tacker.common import utils
 from tacker.extensions import vnfm
+from tacker.plugins.common import constants
 from toscaparser import properties
 from toscaparser.utils import yamlparser
 
@@ -30,6 +33,7 @@ FAILURE = 'tosca.policies.tacker.Failure'
 LOG = logging.getLogger(__name__)
 MONITORING = 'tosca.policies.Monitoring'
 SCALING = 'tosca.policies.Scaling'
+RESERVATION = 'tosca.policies.Reservation'
 PLACEMENT = 'tosca.policies.tacker.Placement'
 TACKERCP = 'tosca.nodes.nfv.CP.Tacker'
 TACKERVDU = 'tosca.nodes.nfv.VDU.Tacker'
@@ -56,6 +60,8 @@ CPU_PROP_MAP = (('hw:cpu_policy', 'cpu_affinity'),
                 ('hw:cpu_threads', 'thread_count'),
                 ('hw:cpu_cores', 'core_count'))
 
+CPU_PROP_VAL_MAP = {'cpu_affinity': ('shared', 'dedicated')}
+
 CPU_PROP_KEY_SET = {'cpu_affinity', 'thread_allocation', 'socket_count',
                     'thread_count', 'core_count'}
 
@@ -81,7 +87,7 @@ deletenodes = (MONITORING, FAILURE, PLACEMENT)
 
 HEAT_RESOURCE_MAP = {
     "flavor": "OS::Nova::Flavor",
-    "image": "OS::Glance::Image"
+    "image": "OS::Glance::WebImage"
 }
 
 SCALE_GROUP_RESOURCE = "OS::Heat::AutoScalingGroup"
@@ -182,50 +188,119 @@ def get_vdu_applicationmonitoring(template):
 
 
 @log.log
-def get_vdu_metadata(template):
+def get_vdu_metadata(template, unique_id=None):
     metadata = dict()
     metadata.setdefault('vdus', {})
     for nt in template.nodetemplates:
         if nt.type_definition.is_derived_from(TACKERVDU):
             metadata_dict = nt.get_property_value('metadata') or None
             if metadata_dict:
+                metadata_dict['metering.server_group'] = \
+                    (metadata_dict['metering.server_group'] + '-'
+                     + unique_id)[:15]
                 metadata['vdus'][nt.name] = {}
                 metadata['vdus'][nt.name].update(metadata_dict)
     return metadata
 
 
 @log.log
-def pre_process_alarm_resources(vnf, template, vdu_metadata):
+def get_metadata_for_reservation(template, metadata):
+    """Method used to add lease_id in metadata
+
+     So that it can be used further while creating query_metadata
+
+    :param template: ToscaTemplate object
+    :param metadata: metadata dict
+    :return: dictionary contains lease_id
+    """
+
+    metadata.setdefault('reservation', {})
+    input_param_list = template.parsed_params.keys()
+    # if lease_id is passed in the parameter file,
+    # get it from template parsed_params.
+    if 'lease_id' in input_param_list:
+        metadata['reservation']['lease_id'] = template.parsed_params[
+            'lease_id']
+    else:
+        for policy in template.policies:
+            if policy.entity_tpl['type'] == constants.POLICY_RESERVATION:
+                metadata['reservation']['lease_id'] = policy.entity_tpl[
+                    'reservation']['properties']['lease_id']
+                break
+    if not uuidutils.is_uuid_like(metadata['reservation']['lease_id']):
+        raise exceptions.Invalid('Invalid UUID for lease_id')
+    return metadata
+
+
+@log.log
+def pre_process_alarm_resources(vnf, template, vdu_metadata, unique_id=None):
     alarm_resources = dict()
-    matching_metadata = dict()
+    query_metadata = dict()
     alarm_actions = dict()
     for policy in template.policies:
-        if (policy.type_definition.is_derived_from(MONITORING)):
-            matching_metadata =\
-                _process_matching_metadata(vdu_metadata, policy)
-            alarm_actions = _process_alarm_actions(vnf, policy)
-    alarm_resources['matching_metadata'] = matching_metadata
+        if policy.type_definition.is_derived_from(MONITORING):
+            query_metadata.update(_process_query_metadata(
+                vdu_metadata, policy, unique_id))
+            alarm_actions.update(_process_alarm_actions(vnf, policy))
+        if policy.type_definition.is_derived_from(RESERVATION):
+            query_metadata.update(_process_query_metadata_reservation(
+                vdu_metadata, policy))
+            alarm_actions.update(_process_alarm_actions_for_reservation(
+                vnf, policy))
+            alarm_resources['event_types'] = {
+                'start_actions': {'event_type': 'lease.event.start_lease'},
+                'before_end_actions': {
+                    'event_type': 'lease.event.before_end_lease'},
+                'end_actions': {'event_type': 'lease.event.end_lease'}}
+    alarm_resources['query_metadata'] = query_metadata
     alarm_resources['alarm_actions'] = alarm_actions
     return alarm_resources
 
 
-def _process_matching_metadata(metadata, policy):
-    matching_mtdata = dict()
+def _process_query_metadata(metadata, policy, unique_id):
+    query_mtdata = dict()
     triggers = policy.entity_tpl['triggers']
     for trigger_name, trigger_dict in triggers.items():
-        if not (trigger_dict.get('metadata') and metadata):
-            raise vnfm.MetadataNotMatched()
-        is_matched = False
-        for vdu_name, metadata_dict in metadata['vdus'].items():
-            if trigger_dict['metadata'] ==\
-                    metadata_dict['metering.vnf']:
-                is_matched = True
-        if not is_matched:
-            raise vnfm.MetadataNotMatched()
-        matching_mtdata[trigger_name] = dict()
-        matching_mtdata[trigger_name]['metadata.user_metadata.vnf'] =\
-            trigger_dict['metadata']
-    return matching_mtdata
+        resource_type = trigger_dict.get('condition').get('resource_type')
+        # TODO(phuoc): currently, Tacker only supports resource_type with
+        # instance value. Other types such as instance_network_interface,
+        # instance_disk can be supported in the future.
+        if resource_type == 'instance':
+            if not (trigger_dict.get('metadata') and metadata):
+                raise vnfm.MetadataNotMatched()
+            is_matched = False
+            for vdu_name, metadata_dict in metadata['vdus'].items():
+                trigger_dict['metadata'] = \
+                    (trigger_dict['metadata'] + '-' + unique_id)[:15]
+                if trigger_dict['metadata'] == \
+                        metadata_dict['metering.server_group']:
+                    is_matched = True
+            if not is_matched:
+                raise vnfm.MetadataNotMatched()
+            query_template = dict()
+            query_template['str_replace'] = dict()
+            query_template['str_replace']['template'] = \
+                '{"=": {"server_group": "scaling_group_id"}}'
+            scaling_group_param = \
+                {'scaling_group_id': trigger_dict['metadata']}
+            query_template['str_replace']['params'] = scaling_group_param
+        else:
+            raise vnfm.InvalidResourceType(resource_type=resource_type)
+        query_mtdata[trigger_name] = query_template
+    return query_mtdata
+
+
+def _process_query_metadata_reservation(metadata, policy):
+    query_metadata = dict()
+    policy_actions = policy.entity_tpl['reservation'].keys()
+    policy_actions.remove('properties')
+    for action in policy_actions:
+        query_template = [{
+            "field": 'traits.lease_id', "op": "eq",
+            "value": metadata['reservation']['lease_id']}]
+        query_metadata[action] = query_template
+
+    return query_metadata
 
 
 def _process_alarm_actions(vnf, policy):
@@ -242,6 +317,20 @@ def _process_alarm_actions(vnf, policy):
     return alarm_actions
 
 
+def _process_alarm_actions_for_reservation(vnf, policy):
+    # process  alarm url here
+    alarm_actions = dict()
+    policy_actions = policy.entity_tpl['reservation'].keys()
+    policy_actions.remove('properties')
+    for action in policy_actions:
+        alarm_url = vnf['attributes'].get(action)
+        if alarm_url:
+            LOG.debug('Alarm url in heat %s', alarm_url)
+            alarm_actions[action] = dict()
+            alarm_actions[action]['alarm_actions'] = [alarm_url]
+    return alarm_actions
+
+
 def get_volumes(template):
     volume_dict = dict()
     node_tpl = template['topology_template']['node_templates']
@@ -329,6 +418,10 @@ def add_resources_tpl(heat_dict, hot_res_tpl):
             }
 
             for prop, val in (vdu_dict).items():
+                # change from 'get_input' to 'get_param' to meet HOT template
+                if isinstance(val, dict):
+                    if 'get_input' in val:
+                        val['get_param'] = val.pop('get_input')
                 heat_dict["resources"][res_name]["properties"][prop] = val
             if heat_dict["resources"].get(vdu):
                 heat_dict["resources"][vdu]["properties"][res] = {
@@ -386,8 +479,8 @@ def represent_odict(dump, tag, mapping, flow_style=None):
 
 @log.log
 def post_process_heat_template(heat_tpl, mgmt_ports, metadata,
-                               alarm_resources, res_tpl,
-                               vol_res={}, unsupported_res_prop=None):
+                               alarm_resources, res_tpl, vol_res={},
+                               unsupported_res_prop=None, unique_id=None):
     #
     # TODO(bobh) - remove when heat-translator can support literal strings.
     #
@@ -409,27 +502,37 @@ def post_process_heat_template(heat_tpl, mgmt_ports, metadata,
         else:
             heat_dict['outputs'] = output
         LOG.debug('Added output for %s', outputname)
-    if metadata:
+    if metadata.get('vdus'):
         for vdu_name, metadata_dict in metadata['vdus'].items():
+            metadata_dict['metering.server_group'] = \
+                (metadata_dict['metering.server_group'] + '-' + unique_id)[:15]
             if heat_dict['resources'].get(vdu_name):
                 heat_dict['resources'][vdu_name]['properties']['metadata'] =\
                     metadata_dict
-    matching_metadata = alarm_resources.get('matching_metadata')
+
+    query_metadata = alarm_resources.get('query_metadata')
     alarm_actions = alarm_resources.get('alarm_actions')
-    if matching_metadata:
-        for trigger_name, matching_metadata_dict in matching_metadata.items():
+    event_types = alarm_resources.get('event_types')
+    if query_metadata:
+        for trigger_name, matching_metadata_dict in query_metadata.items():
             if heat_dict['resources'].get(trigger_name):
-                matching_mtdata = dict()
-                matching_mtdata['matching_metadata'] =\
-                    matching_metadata[trigger_name]
-                heat_dict['resources'][trigger_name]['properties'].\
-                    update(matching_mtdata)
+                query_mtdata = dict()
+                query_mtdata['query'] = \
+                    query_metadata[trigger_name]
+                heat_dict['resources'][trigger_name][
+                    'properties'].update(query_mtdata)
     if alarm_actions:
         for trigger_name, alarm_actions_dict in alarm_actions.items():
             if heat_dict['resources'].get(trigger_name):
                 heat_dict['resources'][trigger_name]['properties']. \
                     update(alarm_actions_dict)
 
+    if event_types:
+        for trigger_name, event_type in event_types.items():
+            if heat_dict['resources'].get(trigger_name):
+                heat_dict['resources'][trigger_name]['properties'].update(
+                    event_type)
+
     add_resources_tpl(heat_dict, res_tpl)
     for res in heat_dict["resources"].values():
         if not res['type'] == HEAT_SOFTWARE_CONFIG:
@@ -474,6 +577,18 @@ def add_volume_resources(heat_dict, vol_res):
 
 @log.log
 def post_process_template(template):
+    def _add_scheduler_hints_property(nt):
+        hints = nt.get_property_value('scheduler_hints')
+        if hints is None:
+            hints = OrderedDict()
+            hints_schema = {'type': 'map', 'required': False,
+                            'entry_schema': {'type': 'string'}}
+            hints_prop = properties.Property('scheduler_hints',
+                                             hints,
+                                             hints_schema)
+            nt.get_properties_objects().append(hints_prop)
+        return hints
+
     for nt in template.nodetemplates:
         if (nt.type_definition.is_derived_from(MONITORING) or
             nt.type_definition.is_derived_from(FAILURE) or
@@ -508,6 +623,43 @@ def post_process_template(template):
                         nt.get_properties_objects().append(newprop)
                         nt.get_properties_objects().remove(p)
 
+        if nt.type_definition.is_derived_from(TACKERVDU):
+            reservation_metadata = nt.get_property_value(
+                'reservation_metadata')
+            if reservation_metadata is not None:
+                hints = _add_scheduler_hints_property(nt)
+
+                input_resource_type = reservation_metadata.get(
+                    'resource_type')
+                input_id = reservation_metadata.get('id')
+
+                # Checking if 'resource_type' and 'id' is passed through a
+                # input parameter file or not. If it's then get the value
+                # from input parameter file.
+                if (isinstance(input_resource_type, OrderedDict) and
+                        input_resource_type.get('get_input')):
+                    input_resource_type = template.parsed_params.get(
+                        input_resource_type.get('get_input'))
+                # TODO(niraj-singh): Remove this validation once bug
+                # 1815755 is fixed.
+                if input_resource_type not in (
+                        'physical_host', 'virtual_instance'):
+                    raise exceptions.Invalid(
+                        'resoure_type must be physical_host'
+                        ' or virtual_instance')
+
+                if (isinstance(input_id, OrderedDict) and
+                        input_id.get('get_input')):
+                    input_id = template.parsed_params.get(
+                        input_id.get('get_input'))
+
+                if input_resource_type == 'physical_host':
+                    hints['reservation'] = input_id
+                elif input_resource_type == 'virtual_instance':
+                    hints['group'] = input_id
+                nt.get_properties_objects().remove(nt.get_properties().get(
+                    'reservation_metadata'))
+
 
 @log.log
 def get_mgmt_driver(template):
@@ -568,8 +720,8 @@ def populate_flavor_extra_specs(es_dict, properties, flavor_extra_input):
                 error_msg_details=(mval + ":Invalid Input"))
         es_dict['hw:mem_page_size'] = mval
     if 'numa_nodes' in properties and 'numa_node_count' in properties:
-        LOG.warning('Both numa_nodes and numa_node_count have been'
-                    'specified; numa_node definitions will be ignored and'
+        LOG.warning('Both numa_nodes and numa_node_count have been '
+                    'specified; numa_node definitions will be ignored and '
                     'numa_node_count will be applied')
     if 'numa_node_count' in properties:
         es_dict['hw:numa_nodes'] = \
@@ -601,8 +753,14 @@ def populate_flavor_extra_specs(es_dict, properties, flavor_extra_input):
                 error_msg_details=(', '.join(invalid_input)),
                 valid_keys=(', '.join(CPU_PROP_KEY_SET)))
         for(k, v) in CPU_PROP_MAP:
-            if v in cpu_dict:
-                es_dict[k] = cpu_dict[v]
+            if v not in cpu_dict:
+                continue
+            if CPU_PROP_VAL_MAP.get(v, None):
+                if cpu_dict[v] not in CPU_PROP_VAL_MAP[v]:
+                    raise vnfm.CpuAllocationInvalidValues(
+                        error_msg_details=cpu_dict[v],
+                        valid_values=CPU_PROP_VAL_MAP[v])
+            es_dict[k] = cpu_dict[v]
     if flavor_extra_input:
         es_dict.update(flavor_extra_input)
 
@@ -668,6 +826,14 @@ def get_nested_resources_name(template):
             return nested_resource_name
 
 
+def get_sub_heat_tmpl_name(template):
+    for policy in template.policies:
+        if (policy.type_definition.is_derived_from(SCALING)):
+            sub_heat_tmpl_name = policy.name + '_' + \
+                uuidutils.generate_uuid() + '_res.yaml'
+            return sub_heat_tmpl_name
+
+
 def update_nested_scaling_resources(nested_resources, mgmt_ports, metadata,
                                     res_tpl, unsupported_res_prop=None):
     nested_tpl = dict()
@@ -676,10 +842,11 @@ def update_nested_scaling_resources(nested_resources, mgmt_ports, metadata,
             list(nested_resources.items())[0]
         nested_resources_dict =\
             yamlparser.simple_ordered_parse(nested_resources_yaml)
-        if metadata:
+        if metadata.get('vdus'):
             for vdu_name, metadata_dict in metadata['vdus'].items():
-                nested_resources_dict['resources'][vdu_name]['properties']['metadata'] = \
-                    metadata_dict
+                if nested_resources_dict['resources'].get(vdu_name):
+                    nested_resources_dict['resources'][vdu_name]['properties']['metadata'] = \
+                        metadata_dict
         add_resources_tpl(nested_resources_dict, res_tpl)
         for res in nested_resources_dict["resources"].values():
             if not res['type'] == HEAT_SOFTWARE_CONFIG:
